﻿using System;
using System.Text;
using System.IO;

namespace AutoDialer
{
    public class cVaxServerExCOM : cVaxServerCOM
    {
        public static cVaxServerExCOM mVaxExCOM;

        int m_nMusicAutoDialer = -1;

        Boolean m_bIsLineAdded = false;
        Boolean m_bPlayWaveConnected = false;

        Boolean m_bDetectDTMF = false;
        Boolean m_bDetectAMD = false;

        Boolean m_bInitialized = false;

        cVaxCallSessionList m_objSessionList = null;
        private MainForm m_objMainForm;

        public const string VAX_ACCOUNT_LINE_SIP = "LineSIP";
        public const string VAX_ACCOUNT_QUEUE_SIP = "QueueInterView";
                
        #region Methods

        public cVaxServerExCOM(MainForm objMainForm)
        {
            mVaxExCOM = this;

            m_objMainForm = objMainForm;
            ResetAll();
        }
        
        ~cVaxServerExCOM()
        {

        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        private void ResetAll()
        {
            m_objSessionList = new cVaxCallSessionList();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool Initialize(int nListenPort)
        {
            UnInitialize();

            base.SetLicenseKey("TRIAL-LICENSE-KEY");

            if (!base.Initialize(""))
                return false;

            if (!base.OpenNetworkUDP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            /*
            if (!base.OpenNetworkTCP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTLS("", nListenPort + 1, ""))
            {
                UnInitialize();
                return false;
            }
            */

            base.SetListenPortRangeRTP(20000, 40000);

            //base.AddNetworkRouteSIP("10.11.0.117", "81.175.35.25");
            //base.AddNetworkRouteRTP("10.11.0.117", "81.175.35.25");
            
            base.AudioSessionLost(true, 20); // Enable voice session lost for 20 seconds.
            base.VideoCOMM(false);

            base.AddQueue(VAX_ACCOUNT_QUEUE_SIP);
            base.SetQueueProcessMode(VAX_ACCOUNT_QUEUE_SIP, VAX_QUEUE_PROCESS_MODE_RING_ALL, VAX_QUEUE_PROCESS_TYPE_FREE);

            for (int nCount = 4000; nCount <= 4080; nCount++)
            {
                base.AddUser(nCount.ToString(), nCount.ToString(), "32");
                base.AddQueueAgent(VAX_ACCOUNT_QUEUE_SIP, nCount.ToString());
            }
                        
            m_bInitialized = true;

            return true;
        }

        public new void UnInitialize()
        {
            m_bInitialized = false;
            UnLoadMusicAutoDialer();
                        
            ResetAll();
                                                
            m_nMusicAutoDialer = -1;
            m_bIsLineAdded = false;

            base.UnInitialize();
        }

        public bool IsInitialized()
        {
            return m_bInitialized;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool RegisterAccountSIP(String sDisplayName, String sUserName, String sAuthUser, String sAuthPwd, String sDomainRealm, String sServerAddr, int nServerPort)
        {   
            if (!base.AddLine(VAX_ACCOUNT_LINE_SIP, VAX_LINE_TYPE_UDP, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sServerAddr, nServerPort, "32"))
                return false;
                        
            if (!RegisterLine(VAX_ACCOUNT_LINE_SIP, 1800))
                return false;

            m_bIsLineAdded = true;

            return true;
        }

        public void UnRegisterAccountSIP()
        {
            base.UnRegisterLine(VAX_ACCOUNT_LINE_SIP);
            base.RemoveLine(VAX_ACCOUNT_LINE_SIP);

            m_bIsLineAdded = false;
        }

        public bool IsAddedAccountSIP()
        {
            return m_bIsLineAdded;
        }

        /////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////

        public void DialCall(string sDialNo, UInt32 nPhoneNoId)
        {
            int nSessionId = (int) DialCallSession("", "", sDialNo, VAX_ACCOUNT_LINE_SIP, 30);
            if (nSessionId == -1) return;

            m_objSessionList.AddSession(nSessionId);

            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.m_sDialedNo = sDialNo;
            objCallSession.m_sStatus = cDefine.PHONE_NO_PROCESSING;

            objCallSession.m_nPhoneNoId = nPhoneNoId;
            objCallSession.m_bDialNoCallType = true;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_PROCESSING, "", "");
        }

        public void CloseCall(UInt32 nPhoneNoId)
        {
            cVaxCallSession objCallSession = m_objSessionList.FindPhoneNoSession(nPhoneNoId);
            if (objCallSession == null) return;


            CloseCallSession(objCallSession.m_nSessionId);
        }
                
        //////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////
                
        public Boolean LoadMusicAutoDialer(string sWaveFile)
        {
            UnLoadMusicAutoDialer();

            if (sWaveFile.Length == 0)
                return false;

            // Load wave file just one-time, don't load it again and again, it affects the performance. Load multiple files on startup.
            
            m_nMusicAutoDialer = base.LoadWaveFile(sWaveFile);

            if (m_nMusicAutoDialer == -1)
                return false;

            return true;
        }

        public  void UnLoadMusicAutoDialer()
        {
            if (m_nMusicAutoDialer == -1)
                return;

            base.UnLoadWaveID(m_nMusicAutoDialer);
            
            m_nMusicAutoDialer = -1;
        }

        public void PlayMusicAutoDialer(int nSessionId)
        {
            if (m_nMusicAutoDialer == -1)
                return;

            base.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, m_nMusicAutoDialer, true, 10);
        }

        public void PlayTextAutoDialer(int nSessionId, int nChannelId, String sText)
        {
            Byte[] aDataPCM = null;
            long nDataSize = 0;

            cTextToSpeech objSpeech = new cTextToSpeech();
            objSpeech.GenSpeech(sText, ref aDataPCM, ref nDataSize);

            base.SendAudioPCM(nSessionId, nChannelId, aDataPCM, (int)nDataSize);
        }

        public void EnableDetectDTMF(bool bDetectDTMF)
        {
            m_bDetectDTMF = bDetectDTMF;
        }

        public void EnableDetectAMD(bool bDetectAMD)
        {
            m_bDetectAMD = bDetectAMD;
        }
                
        public void SetPlayWaveConnected(bool bPlayWaveConnected)
        {
            m_bPlayWaveConnected = bPlayWaveConnected;
        }

       //////////////////////////////////////////////////////////////////////////////////
       //////////////////////////////////////////////////////////////////////////////////
        
        #endregion
       
        #region Events

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            m_objSessionList.AddSession(nSessionId);
        }

        protected override void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            if (objCallSession.IsConnected())
            {
                objCallSession.EndCall();
                String sDuration = objCallSession.GetDuration();

                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, "", "", sDuration);
            }

            objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode);

            m_objSessionList.RemoveSession(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionAccessAudioPCM(int nSessionId, int nChannelId, int nAccessType, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionConnected(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;
            
            if (objCallSession.m_bDialNoCallType == false)
                return;

            objCallSession.OnCallSessionConnected(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE);

            if (m_bPlayWaveConnected)
                PlayMusicAutoDialer(nSessionId);

            if (m_bDetectAMD)
                base.CallSessionDetectAMD(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, true, 5000, 300, 2);  //5-Seconds, 300-MilliSeconds, 2-Silences

            if (m_bDetectDTMF)
            {
                base.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, cVaxServerCOM.VAX_DTMF_TYPE_RFC2833, true, 2000);  // Enable DTMF detection RFC2833 based.
                base.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, cVaxServerCOM.VAX_DTMF_TYPE_SIP_INFO, true, 2000);     // Enable DTMF detection SIP INFO based.
            }
                        
            objCallSession.StartCall();
            objCallSession.SetConnected();

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_CONNECTED, "", "");

            if (objCallSession.m_bDialNoCallType)
            {
                base.AddCallSessionToQueue(VAX_ACCOUNT_QUEUE_SIP, "", "", nSessionId, 40);
            }
        }
                             
        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnDetectedDigitDTMF(int nSessionId, int nChannelId, String sDigitDTMF, int nTypeDTMF)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, "", sDigitDTMF, "");

            string sMsgDTMF = "Digit Detected: [ " + sDigitDTMF + " ]";

            PlayTextAutoDialer(nSessionId, nChannelId, sMsgDTMF);

            GenStatusLog(sMsgDTMF);
        }

        protected override void OnCallSessionFailed(int nSessionId, int nChannelId, int nStatusCode, String sReasonPhrase, String sContact)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            string sStatus;
            sStatus = cDefine.PHONE_NO_FAILED + " : " +nStatusCode.ToString() + "-" + sReasonPhrase;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, sStatus, "", "");
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionTimeout(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_TIMEOUT, "", "");
        }

        protected override void OnCallSessionCancelled(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_CANCELED, "", "");
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionDetectedAMD(int nSessionId, int nChannelId, Boolean bIsHuman)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            if (bIsHuman)
            {
                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.AMD_HUMAN_VOICE, "", "");
                return;
            }

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.AMD_MACHINE_VOICE, "", "");
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionMediaDetectStartedVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId);
        }
        protected override void OnCallSessionMediaDetectCompletedVAD(int nSessionId, int nChannelId, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM);
        }

        protected override void OnCallSessionMediaDetectExpiredVAD(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnRegisterUser(string sUserLogin, string sDomain, string sUserAgentName, string sFromIP, int nFromPort, int nRegId)
        {
            base.AuthRegister(nRegId);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////
        
        protected override void OnLineRegisterTrying(string sLineName)
        {
            string sStatus = sLineName + " " + cDefine.LINE_REG_TRYING;
            GenStatusLog(sStatus);
        }

        protected override void OnLineRegisterFailed(string sLineName, int nStatusCode, string sReasonPhrase)
        {
            string sStatus = sLineName + " Register Failed: " + sReasonPhrase;
            GenStatusLog(sStatus);
        }

        protected override void OnLineRegisterSuccess(string sLineName)
        {
            string sStatus = sLineName + " " + cDefine.LINE_REG_SUCCESS;
            GenStatusLog(sStatus);
        }
                        
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(string sStatus)
        {
            m_objMainForm.OnVaxStatusLog(sStatus);
        }

        protected override void OnVaxErrorLog(String sFuncName, int nErrorCode, String sErrorMsg)
        {
            string sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            GenStatusLog(sMsg);
        }

        protected override void OnCallSessionErrorLog(int nSessionId, int nChannelId, int nErrorCode, String sErrorMsg)
        {
            string sMsg = "[Session: " + nSessionId.ToString() + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            GenStatusLog(sMsg);
        }
                
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnOutgoingDiagnosticLog(string sMsgSIP, string sToIP, int nToPort)
        {
            FileStream LogFile = new FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite);
            LogFile.Seek(0, SeekOrigin.End);

            String sNewLine =  "\r\n";
            byte[] aNewLine = Encoding.ASCII.GetBytes(sNewLine);
            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sAddressInfo = String.Format("{0}:{1}", sToIP, nToPort);
            byte[] aAddressInfo = Encoding.ASCII.GetBytes(sAddressInfo);
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sLogType = "============== OUTGOING============";
            byte[] aLogType = Encoding.ASCII.GetBytes(sLogType);
            LogFile.Write(aLogType, 0, sLogType.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            byte[] aMsgSIP = Encoding.ASCII.GetBytes(sMsgSIP);
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length);

            LogFile.Close();
        }

        protected override void OnIncomingDiagnosticLog(string sMsgSIP, string sFromIP, int nFromPort)
        {
            FileStream LogFile = new FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite);
            LogFile.Seek(0, SeekOrigin.End);

            String sNewLine = "\r\n";
            byte[] aNewLine = Encoding.ASCII.GetBytes(sNewLine);
            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sAddressInfo = String.Format("{0}:{1}", sFromIP, nFromPort);
            byte[] aAddressInfo = Encoding.ASCII.GetBytes(sAddressInfo);
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            String sLogType = "============== INCOMING ============";
            byte[] aLogType = Encoding.ASCII.GetBytes(sLogType);
            LogFile.Write(aLogType, 0, sLogType.Length);

            LogFile.Write(aNewLine, 0, sNewLine.Length);

            byte[] aMsgSIP = Encoding.ASCII.GetBytes(sMsgSIP);
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length);

            LogFile.Close();
        }

                
        #endregion

    }
}
